#!/usr/bin/perl
#
# Ein simpler Portscanner, der offene bzw. geschlossene Ports
# mittels TCP / IP Packete, die nur das URG Bit gesetzt haben
# ermittelt.
# Open Port send ACK
# Closed Port RST
# Keine Antwort nach Timeout --> Packet wurde vermutlich von einem
# Packetfilter gedroppt
#
# C0D3D by <<<Bytebeater>>> [ bytebeater@crazydj.de ]
# http://www.crazydj.de
#
# Last Update: 15.03.2002
#
# Perl is phun!!! =)

###[ Loading Modules ]###

use Net::RawIP;                      # Generate IP packets
use Net::PcapUtils;		     # Libpcap Interface
use NetPacket::Ethernet qw(:strip);  # Decode Ethernet packets	
use NetPacket::IP qw(:strip);	     # Decode IP packets
use NetPacket::TCP;		     # Decode TCP packets
use Getopt::Std;		     # Parsing parameter
use strict;			     # Sei streng mit dem Programmierer


###[ Configuration ]###

# Network Device
my $dev = "eth0";

# Spoofed source IP address
my $ipaddr = '192.168.3.33';

# Source Port
# Auf diesem Port warten wir auf Antworten
my $sourceport = '7777';

# Wie lange soll das Tool auf eine Antwort
# warten bis es annimmt, dass das Packet
# gedroppt wurde?
# Angabe in Millisekunden
my $timeout = 3000;


# Run in Debug mode?
# 0 == Off
# 1 == On
my $debug = 1;



###[ Globale Variablen ]###

# Parameter
my %args;

# Liste aller Targets und Ports
my(@target,@port);

# Das Ergebnis des Scans wird in einem Hash gesammelt
my %ergebnis;





###[ MAIN PART ]###

# Parsing parameter
getopts('t:p:h',\%args);

# User braucht Hilfe!
if($args{'h'})
{
    &help;
}

unless($args{'t'})
{
    die ">>> I need a target!!!\n\n";
}

my $target = $args{'t'};
my $port = $args{'p'};


# Target list
    if($target =~ /\,/g)
    {
	@target = split(/\,/,$target);
    }

# Target range
    elsif($target =~ /\-/)
    {
	my ($start,$stop) = split(/\-/,$target);

	&check_ip($start);
	&check_ip($stop);

# Rechne die IP in eine Dezimalzahl um
	my @start = split(/\./,$start); 
	my @stop = split(/\./,$stop); 

	my $start_dec = ((@start[0]*16777216)+(@start[1]*65536)+(@start[2]*256)+@start[3]);
	my $stop_dec = ((@stop[0]*16777216)+(@stop[1]*65536)+(@stop[2]*256)+@stop[3]);

	while($start_dec < $stop_dec + 1)
	{
		my @bytez;
		my $rem;

# Rechne die Dezimal Zahl wieder in eine IP Adresse um
	    	@bytez[0] = int $start_dec/16777216;
	    	$rem = $start_dec % 16777216;
	    	@bytez[1] = int $rem/65536;
	    	$rem = $rem % 65536;
	    	@bytez[2] = int $rem/256;
	    	$rem = $rem % 256;
	    	@bytez[3] = $rem;
	  	my $ipaddr = join '.', @bytez;
		push @target, $ipaddr;
		$start_dec++;
	}
     }
     # Only one target
     else
     {
	 print "Single target.\n" if $debug;
	 &check_ip($target);
	 push @target,$target;
     }




# Falls kein Port angegeben wurde, scanner wir alle unter 1024
if($port eq "")
{
    for(my $i=1; $i<1025;$i++)
    {
	push @port,$i;
    }
}
else
{
	# Port List?
	if($port =~ /\,/)
	{
		@port = split(/\,/,$port);
	}

	# Port Range?
	elsif($port =~ /\-/)
	{
		my($start,$stop) = split(/\-/,$port);

		for(my $i=$start; $i <= $stop; $i++)
		{
			push @port, $i;
		}
	}
	# Only one port
	else
	{
	    push @port,$port;
	}
}




###[ Start scanning ]###

print "Start URG scanning...\n\n";

# Generiere fuer jeden Target und jeden Port ein URG Packet
foreach my $host (@target)
{
print "Target: $host  ";

    for(@port)
    {
	my $packet = new Net::RawIP;

	$packet->set({
	    		ip => {
				saddr => $ipaddr,
				daddr => $host
			       },
			tcp => { 
				source => $sourceport,
				dest => $_,
				urg => '1',
				window => '2323'
				}
			});


	print "URG Packet send to $host $_\n";

	my $userdata = [ $host, $_ ];

	# Warte $timeout Sekunden auf eine Antwort
	Net::PcapUtils::loop(\&check_dump,
			     PROMISC => 1,
			     FILTER => 'tcp',
			     DEV => $dev,
			     TIMEOUT => 3000,
			     USERDATA => $userdata);				

	}

print "\n";
}



###[ Subroutines ]###

# Check the sniffed packets.
# Ist da ne Antwort fuer uns dabei??
sub check_dump
{
my($arg, $header, $packet) = @_;

my($host,$port) = @{$arg};

# Decode the packet
  my $ip = NetPacket::IP->decode(eth_strip($packet));
  my $tcp = NetPacket::TCP->decode($ip->{data});


# Ist das Packet eine Antwort auf unser Scan Packet?
if(($tcp->{window} == 2323) && ($ip->{daddr} eq $ipaddr) && ($ip->{saddr} eq $host) && ($tcp->{sport} == $port) && ($tcp->{dport} == $sourceport))
{
	print "Got a response!\n" if $debug;
	return 1;
}
else
{
	print $ip->{src_ip} . ":" . $tcp->{src_port} . "  -->  " . $ip->{dest_ip} . ":" . $tcp->{dest_port} . "\n";
	return 0;
}
}



# Check if ip address is correct
sub check_ip
{
    my $ip = shift;

    my @ip = split(/\./,$ip);

if(scalar(@ip != 4))
{
    die ">>>  Bad IP address $ip!\n\n";
}

for(@ip)
{
    if($_ > 256)
    {
	die ">>> Bad IP address $ip!\n\n";
    }
}
}


# Print Usage
sub help
{
    print "Usage: $0 -t <target> -p <port>\n";
    print "Target and port can either be a range like 23-25 or\n";
    print "a list like 23,24,25\n\n";
    exit(0);
}
