package config;

# Check Configuration
#
# Written by Bastian Ballmann [ bytebeater@crazydj.de ]

# Create a new config object
sub new
{
    my $class = shift;
    my $obj{'target'} = [];
    $obj{'connection'} = [];
    $obj{'port'} = [];
    $obj{'multitarget'} = 0;
    $obj{'multiport'} = 0;
    $obj{'device'} = "eth0";
    $obj = bless %obj, $class;
    return $obj;
}

# Check the configuration
sub check
{
    my $obj = shift;
    my %args = %{shift};

# Die Connection soll fuer ein oder mehrere Hosts resettet werden
if($args{'h'})
{
    $target = $args{'h'};

# Target list
    if($target =~ /\,/g)
    {
	$obj->{'target'} = split(/\,/,$target);
	$obj->{'multitarget'} = 1;
    }

# Target range
    elsif($target =~ /\-/)
    {
	my ($start,$stop) = split(/\-/,$target);

	&check_ip($start);
	&check_ip($stop);

# Rechne die IP in eine Dezimalzahl um
	my @start = split(/\./,$start); 
	my @stop = split(/\./,$stop); 

	my $start_dec = ((@start[0]*16777216)+(@start[1]*65536)+(@start[2]*256)+@start[3]);
	my $stop_dec = ((@stop[0]*16777216)+(@stop[1]*65536)+(@stop[2]*256)+@stop[3]);


# Berechne Start / End IP
	while($start_dec < $stop_dec + 1)
	{
		my @bytez;
		my $rem;

# Rechne die Dezimal Zahl wieder in eine IP Adresse um
	    	@bytez[0] = int $start_dec/16777216;
	    	$rem = $start_dec % 16777216;
	    	@bytez[1] = int $rem/65536;
	    	$rem = $rem % 65536;
	    	@bytez[2] = int $rem/256;
	    	$rem = $rem % 256;
	    	@bytez[3] = $rem;
	  	my $ipaddr = join '.', @bytez;
		push $obj->{'target'}, $ipaddr;
		$start_dec++;
	}
	
	$obj->{'multitarget'} = 1;
    }

# Reset eine einzelne Connection
    else
    {
	&check_ip($target);
    }

}

# Wir sollen nur Packete fuer einen bestimmten Port resetten
if($args{'p'})
{
    $port = $args{'p'};

	# Port List?
	if($port =~ /\,/)
	{
		@{$obj->{'port'}} = split(/\,/,$port);
		$obj->{'multiport'} = 1;
	}

	# Port Range?
	if($port =~ /\-/)
	{
		my($start,$stop) = split(/\-/,$port);

		for(my $i=$start; $i <= $stop; $i++)
		{
			push $obj->{'port'}, $i;
		}

		$obj->{'multiport'} = 1;
	}
}

return $obj;
}


# Print configuration
sub print
{
    my $obj = shift;

# Device 
    print "---:[ Device $obj->{'device'} ]:---\n\n";

# Port Config
    if(defined $obj->{'port'})
    {
	if($obj->multiport())
	{
	    print "---:[ Ports ";

	    for(@{$obj->{'port'}})
	    {
		print "$_ ";
	    }

	    print "]:---\n\n";
	}
	else
	{
	    print "---:[ Port $obj->{'port'}->[0] ]:---\n\n";
	}
    }


# Host Config
    if(defined $obj->{'host'})
    {
	if($obj->multitarget())
	{
	    print "---:[ Hosts ";

	    for(@{$obj->{'host'}})
	    {
		print "$_ ";
	    }

	    print "]:---\n\n";
	}
	else
	{
	    print "---:[ Host $obj->{'host'}->[0] ]:---\n\n";
	}
    }

}


# Change host
sub set_host
{
    my $obj = shift;
    my @host = @_;
    @{$obj->{'host'}} = @host;
    return $obj;
}


# Change connection
sub set_connection
{
    my $obj = shift;
    my $connection = shift;
    $obj->{'connection'} = $connection;
    return $obj;
}

# Change port
sub set_port
{
    my $obj = shift;
    my @port = @_;
    @{$obj->{'port'}} = @port;
    return $obj;
}


# Change device
sub set_device
{
    my $obj = shift;
    my $device = shift;
    $obj->{'device'} = $device;
    return $device;
}

# Extract target host(s)
sub get_host
{
    my $obj = shift;
    return $obj->{'target'};
}


# Extract connection(s)
sub get_connection
{
    my $obj = shift;
    return $obj->{'connection'};
}


# Extract port(s)
sub get_port
{
    my $obj = shift;
    return $obj->{'port'};
}

# Extract device
sub get_device
{
    my $obj = shift;
    return $obj->{'device'};
}

# Multitarget?
sub multitarget
{
    my $obj = shift;
    return $obj->{'multitarget'};
}


# Multi port?
sub multiport
{
    my $obj = shift;
    return $obj->{'multiport'};
}


# Check if ip address is correct
sub check_ip
{
    my $ip = shift;

    my @ip = split(/\./,$ip);

# Mehr als 4 Byte?
if(scalar(@ip != 4))
{
    die ">>>  Bad IP address $ip!\n\n";
}

# Ist ein Wert groesser als 256?
for(@ip)
{
    if($_ > 256)
    {
	die ">>> Bad IP address $ip!\n\n";
    }
}
}





1;
